@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -29,7 +29,6 @@
  * @brief Class @ref Magnum::GL::AbstractShaderProgram
  */
 
-#include <functional>
 #include <string>
 #include <Corrade/Containers/ArrayView.h>
 
@@ -55,8 +54,8 @@ This class is designed to be used via subclassing. Subclasses define these
 functions and properties:
 
 <ul>
-<li> **Attribute definitions** with location and type for
-    configuring meshes, for example:
+<li> **Attribute definitions** using @ref Attribute typedefs with location and
+    type for configuring meshes, for example:
 
     @snippet MagnumGL.cpp AbstractShaderProgram-input-attributes
 </li>
@@ -89,7 +88,7 @@ functions and properties:
     @snippet MagnumGL.cpp AbstractShaderProgram-xfb
 </li></ul>
 
-@subsection GL-AbstractShaderProgram-attribute-location Binding attribute location
+@subsection GL-AbstractShaderProgram-attribute-location Binding attribute and fragment data location
 
 The preferred workflow is to specify attribute location for vertex shader input
 attributes and fragment shader output attributes explicitly in the shader code,
@@ -152,7 +151,7 @@ out vec3 normal;
     1.0, similar functionality is available in extension
     @webgl_extension{WEBGL,draw_buffers}.
 
-@todo @gl_extension2{EXT,separate_shader_objects,separate_shader_objects.gles}
+@todo @gl_extension2{EXT,separate_shader_objects,EXT_separate_shader_objects.gles}
     supports explicit attrib location
 
 @subsection GL-AbstractShaderProgram-uniform-location Uniform locations
@@ -407,10 +406,10 @@ are cached, so repeated queries don't result in repeated @fn_gl{Get} calls. See
 also @ref Context::resetState() and @ref Context::State::Shaders.
 
 If extension @gl_extension{ARB,separate_shader_objects} (part of OpenGL 4.1),
-@gl_extension{EXT,direct_state_access} desktop extension, @gl_extension2{EXT,separate_shader_objects,separate_shader_objects.gles} OpenGL ES
-extension or OpenGL ES 3.1 is available, uniform setting functions use DSA
-functions to avoid unnecessary calls to @fn_gl{UseProgram}. See @ref setUniform()
-documentation for more information.
+@gl_extension2{EXT,separate_shader_objects,EXT_separate_shader_objects.gles}
+OpenGL ES extension or OpenGL ES 3.1 is available, uniform setting functions
+use DSA functions to avoid unnecessary calls to @fn_gl{UseProgram}. See
+@ref setUniform() documentation for more information.
 
 To achieve least state changes, set all uniforms in one run --- method chaining
 comes in handy.
@@ -651,6 +650,8 @@ class MAGNUM_GL_EXPORT AbstractShaderProgram: public AbstractObject {
          *
          * This function can be safely used for constructing (and later
          * destructing) objects even without any OpenGL context being active.
+         * However note that this is a low-level and a potentially dangerous
+         * API, see the documentation of @ref NoCreate for alternatives.
          * @see @ref AbstractShaderProgram()
          */
         explicit AbstractShaderProgram(NoCreateT) noexcept;
@@ -757,7 +758,7 @@ class MAGNUM_GL_EXPORT AbstractShaderProgram: public AbstractObject {
          *      @def_gl{LINK_STATUS} and @def_gl{INFO_LOG_LENGTH},
          *      @fn_gl_keyword{GetProgramInfoLog}
          */
-        static bool link(std::initializer_list<std::reference_wrapper<AbstractShaderProgram>> shaders);
+        static bool link(std::initializer_list<Containers::Reference<AbstractShaderProgram>> shaders);
 
         #if !defined(MAGNUM_TARGET_GLES2) && !defined(MAGNUM_TARGET_WEBGL)
         /**
@@ -784,7 +785,7 @@ class MAGNUM_GL_EXPORT AbstractShaderProgram: public AbstractObject {
          * @see @fn_gl_keyword{ProgramParameter} with @def_gl{PROGRAM_SEPARABLE}
          * @requires_gl41 Extension @gl_extension{ARB,separate_shader_objects}
          * @requires_es_extension Extension
-         *      @gl_extension2{EXT,separate_shader_objects,separate_shader_objects.gles}
+         *      @gl_extension2{EXT,separate_shader_objects,EXT_separate_shader_objects.gles}
          * @requires_gles Separate shader objects are not supported in WebGL.
          */
         void setSeparable(bool enabled) {
@@ -797,7 +798,7 @@ class MAGNUM_GL_EXPORT AbstractShaderProgram: public AbstractObject {
         #endif
 
         /**
-         * @brief Attach shader
+         * @brief Attach a shader
          *
          * @see @fn_gl_keyword{AttachShader}
          */
@@ -810,10 +811,10 @@ class MAGNUM_GL_EXPORT AbstractShaderProgram: public AbstractObject {
          * than one shader at once. Other than that there is no other
          * (performance) difference when using this function.
          */
-        void attachShaders(std::initializer_list<std::reference_wrapper<Shader>> shaders);
+        void attachShaders(std::initializer_list<Containers::Reference<Shader>> shaders);
 
         /**
-         * @brief Bind attribute to given location
+         * @brief Bind an attribute to given location
          * @param location      Location
          * @param name          Attribute name
          *
@@ -930,11 +931,11 @@ class MAGNUM_GL_EXPORT AbstractShaderProgram: public AbstractObject {
          * @brief Link the shader
          *
          * Links single shader. If possible, prefer to link multiple shaders
-         * at once using @ref link(std::initializer_list<std::reference_wrapper<AbstractShaderProgram>>)
+         * at once using @ref link(std::initializer_list<Containers::Reference<AbstractShaderProgram>>)
          * for improved performance, see its documentation for more
          * information.
          */
-        bool link() { return link({*this}); }
+        bool link();
 
         /**
          * @brief Get uniform location
@@ -1026,15 +1027,13 @@ class MAGNUM_GL_EXPORT AbstractShaderProgram: public AbstractObject {
          * @param location      Uniform location
          * @param values        Values
          *
-         * If neither @gl_extension{ARB,separate_shader_objects} (part of OpenGL
-         * 4.1) nor @gl_extension{EXT,direct_state_access} desktop extension nor
-         * @gl_extension{EXT,separate_shader_objects} OpenGL ES extension nor
-         * OpenGL ES 3.1 is available, the shader is marked for use before the
-         * operation.
+         * If neither @gl_extension{ARB,separate_shader_objects} (part of
+         * OpenGL 4.1) nor @gl_extension{EXT,separate_shader_objects} OpenGL ES
+         * extension nor OpenGL ES 3.1 is available, the shader is marked for
+         * use before the operation.
          * @see @ref setUniform(Int, const T&), @ref uniformLocation(),
          *      @fn_gl{UseProgram}, @fn_gl_keyword{Uniform} or
-         *      @fn_gl_keyword{ProgramUniform} /
-         *      @fn_gl_extension_keyword{ProgramUniform,EXT,direct_state_access}.
+         *      @fn_gl_keyword{ProgramUniform}
          */
         void setUniform(Int location, Containers::ArrayView<const Float> values);
         void setUniform(Int location, Containers::ArrayView<const Math::Vector<2, Float>> values); /**< @overload */
@@ -1135,8 +1134,10 @@ class MAGNUM_GL_EXPORT AbstractShaderProgram: public AbstractObject {
         #endif
 
         void bindAttributeLocationInternal(UnsignedInt location, Containers::ArrayView<const char> name);
+        #ifndef MAGNUM_TARGET_GLES
         void bindFragmentDataLocationIndexedInternal(UnsignedInt location, UnsignedInt index, Containers::ArrayView<const char> name);
         void bindFragmentDataLocationInternal(UnsignedInt location, Containers::ArrayView<const char> name);
+        #endif
         Int uniformLocationInternal(Containers::ArrayView<const char> name);
         UnsignedInt uniformBlockIndexInternal(Containers::ArrayView<const char> name);
 
@@ -1150,9 +1151,8 @@ class MAGNUM_GL_EXPORT AbstractShaderProgram: public AbstractObject {
         void use();
 
         /*
-            Currently, there are four supported ways to call glProgramUniform():
+            Currently, there are three supported ways to call glProgramUniform():
 
-            - EXT_direct_state_access (desktop GL only, EXT suffix)
             - EXT_separate_shader_objects (OpenGL ES extension, EXT suffix)
             - ARB_separate_shader_objects (desktop GL only, no suffix)
             - OpenGL ES 3.1, no suffix
@@ -1160,14 +1160,12 @@ class MAGNUM_GL_EXPORT AbstractShaderProgram: public AbstractObject {
             To avoid copypasta and filesize bloat, this is merged to just two
             variants of implementation functions:
 
-            - uniformImplementationSSO() - functions without suffix, used if
+            - uniformImplementationSSO() -- functions without suffix, used if
                 ARB_separate_shader_objects desktop extension or OpenGL ES 3.1
                 is available, completely disabled for ES2
-            - uniformImplementationDSAEXT_SSOEXT() / uniformImplementationDSAEXT() --
-                functions with EXT suffix, used if EXT_direct_state_access
-                desktop exttension or EXT_separate_shader_objects ES 2.0 / ES 3.0
-                extension is available, functions which don't have equivalents
-                on ES (double arguments) don't have the _SSOEXT suffix
+            - uniformImplementationSSOEXT() -- functions with EXT suffix, used
+                if EXT_separate_shader_objects ES 2.0 / ES 3.0 extension is
+                available
         */
 
         void MAGNUM_GL_LOCAL uniformImplementationDefault(GLint location, GLsizei count, const GLfloat* values);
@@ -1210,28 +1208,22 @@ class MAGNUM_GL_EXPORT AbstractShaderProgram: public AbstractObject {
         void MAGNUM_GL_LOCAL uniformImplementationSSO(GLint location, GLsizei count, const Math::Vector<3, GLdouble>* values);
         void MAGNUM_GL_LOCAL uniformImplementationSSO(GLint location, GLsizei count, const Math::Vector<4, GLdouble>* values);
         #endif
-        #ifndef MAGNUM_TARGET_WEBGL
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const GLfloat* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::Vector<2, GLfloat>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::Vector<3, GLfloat>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::Vector<4, GLfloat>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const GLint* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::Vector<2, GLint>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::Vector<3, GLint>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::Vector<4, GLint>* values);
+        #if defined(MAGNUM_TARGET_GLES) && !defined(MAGNUM_TARGET_WEBGL)
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const GLfloat* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::Vector<2, GLfloat>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::Vector<3, GLfloat>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::Vector<4, GLfloat>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const GLint* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::Vector<2, GLint>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::Vector<3, GLint>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::Vector<4, GLint>* values);
         #ifndef MAGNUM_TARGET_GLES2
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const GLuint* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::Vector<2, GLuint>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::Vector<3, GLuint>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::Vector<4, GLuint>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const GLuint* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::Vector<2, GLuint>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::Vector<3, GLuint>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::Vector<4, GLuint>* values);
         #endif
         #endif
-        #ifndef MAGNUM_TARGET_GLES
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT(GLint location, GLsizei count, const GLdouble* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT(GLint location, GLsizei count, const Math::Vector<2, GLdouble>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT(GLint location, GLsizei count, const Math::Vector<3, GLdouble>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT(GLint location, GLsizei count, const Math::Vector<4, GLdouble>* values);
-        #endif
 
         void MAGNUM_GL_LOCAL uniformImplementationDefault(GLint location, GLsizei count, const Math::RectangularMatrix<2, 2, GLfloat>* values);
         void MAGNUM_GL_LOCAL uniformImplementationDefault(GLint location, GLsizei count, const Math::RectangularMatrix<3, 3, GLfloat>* values);
@@ -1277,30 +1269,19 @@ class MAGNUM_GL_EXPORT AbstractShaderProgram: public AbstractObject {
         void MAGNUM_GL_LOCAL uniformImplementationSSO(GLint location, GLsizei count, const Math::RectangularMatrix<3, 4, GLdouble>* values);
         void MAGNUM_GL_LOCAL uniformImplementationSSO(GLint location, GLsizei count, const Math::RectangularMatrix<4, 3, GLdouble>* values);
         #endif
-        #ifndef MAGNUM_TARGET_WEBGL
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<2, 2, GLfloat>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<3, 3, GLfloat>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<4, 4, GLfloat>* values);
+        #if defined(MAGNUM_TARGET_GLES) && !defined(MAGNUM_TARGET_WEBGL)
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<2, 2, GLfloat>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<3, 3, GLfloat>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<4, 4, GLfloat>* values);
         #ifndef MAGNUM_TARGET_GLES2
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<2, 3, GLfloat>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<3, 2, GLfloat>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<2, 4, GLfloat>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<4, 2, GLfloat>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<3, 4, GLfloat>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT_SSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<4, 3, GLfloat>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<2, 3, GLfloat>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<3, 2, GLfloat>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<2, 4, GLfloat>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<4, 2, GLfloat>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<3, 4, GLfloat>* values);
+        void MAGNUM_GL_LOCAL uniformImplementationSSOEXT(GLint location, GLsizei count, const Math::RectangularMatrix<4, 3, GLfloat>* values);
         #endif
         #endif
-        #ifndef MAGNUM_TARGET_GLES
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT(GLint location, GLsizei count, const Math::RectangularMatrix<2, 2, GLdouble>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT(GLint location, GLsizei count, const Math::RectangularMatrix<3, 3, GLdouble>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT(GLint location, GLsizei count, const Math::RectangularMatrix<4, 4, GLdouble>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT(GLint location, GLsizei count, const Math::RectangularMatrix<2, 3, GLdouble>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT(GLint location, GLsizei count, const Math::RectangularMatrix<3, 2, GLdouble>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT(GLint location, GLsizei count, const Math::RectangularMatrix<2, 4, GLdouble>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT(GLint location, GLsizei count, const Math::RectangularMatrix<4, 2, GLdouble>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT(GLint location, GLsizei count, const Math::RectangularMatrix<3, 4, GLdouble>* values);
-        void MAGNUM_GL_LOCAL uniformImplementationDSAEXT(GLint location, GLsizei count, const Math::RectangularMatrix<4, 3, GLdouble>* values);
-        #endif
 
         GLuint _id;
 
@@ -1311,17 +1292,6 @@ class MAGNUM_GL_EXPORT AbstractShaderProgram: public AbstractObject {
         #endif
 };
 
-}
-
-#ifdef MAGNUM_BUILD_DEPRECATED
-/* Note: needs to be prefixed with Magnum:: otherwise Doxygen can't find it */
-
-/** @brief @copybrief GL::AbstractShaderProgram
- * @deprecated Use @ref GL::AbstractShaderProgram instead.
- */
-typedef CORRADE_DEPRECATED("use GL::AbstractShaderProgram instead") Magnum::GL::AbstractShaderProgram AbstractShaderProgram;
-#endif
-
-}
+}}
 
 #endif
