@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
     Copyright © 2016, 2018 Jonathan Hale <squareys@googlemail.com>
 
@@ -236,11 +236,9 @@ template<class T> bool pointCone(const Vector3<T>& point, const Vector3<T>& cone
 @return @cpp true @ce if the point is inside the cone, @cpp false @ce
     otherwise
 
-The @p tanAngleSqPlusOne parameter can be calculated as:
+The @p tanAngleSqPlusOne parameter can be precomputed like this:
 
-@code{.cpp}
-Math::pow<2>(Math::tan(angle*T(0.5))) + T(1)
-@endcode
+@snippet MagnumMath.cpp Intersection-tanAngleSqPlusOne
 */
 template<class T> bool pointCone(const Vector3<T>& point, const Vector3<T>& coneOrigin, const Vector3<T>& coneNormal, T tanAngleSqPlusOne);
 
@@ -269,9 +267,7 @@ template<class T> bool pointDoubleCone(const Vector3<T>& point, const Vector3<T>
 
 The @p tanAngleSqPlusOne parameter can be precomputed like this:
 
-@code{.cpp}
-T tanAngleSqPlusOne = Math::pow<2>(Math::tan(angle*T(0.5))) + T(1);
-@endcode
+@snippet MagnumMath.cpp Intersection-tanAngleSqPlusOne
 */
 template<class T> bool pointDoubleCone(const Vector3<T>& point, const Vector3<T>& coneOrigin, const Vector3<T>& coneNormal, T tanAngleSqPlusOne);
 
@@ -301,12 +297,9 @@ template<class T> bool sphereConeView(const Vector3<T>& sphereCenter, T sphereRa
 
 Transforms the sphere center into cone space (using the cone view matrix) and
 performs sphere-cone intersection with the zero-origin -Z axis-aligned cone.
-The @p sinAngle, @p cosAngle, @p tanAngle can be precomputed like this:
+The @p sinAngle and @p tanAngle can be precomputed like this:
 
-@code{.cpp}
-T sinAngle = Math::sin(angle*T(0.5));
-T tanAngle = Math::tan(angle*T(0.5));
-@endcode
+@snippet MagnumMath.cpp Intersection-sinAngle-tanAngle
 */
 template<class T> bool sphereConeView(const Vector3<T>& sphereCenter, T sphereRadius, const Matrix4<T>& coneView, T sinAngle, T tanAngle);
 
@@ -345,10 +338,7 @@ normal direction), and behind the plane, where the test is equivalent to
 testing whether the origin of the original cone intersects the sphere. The
 @p sinAngle and @p tanAngleSqPlusOne parameters can be precomputed like this:
 
-@code{.cpp}
-T sinAngle = Math::sin(angle*T(0.5));
-T tanAngleSqPlusOne = Math::pow<2>(Math::tan(angle*T(0.5))) + T(1);
-@endcode
+@snippet MagnumMath.cpp Intersection-sinAngle-tanAngleSqPlusOne
 */
 template<class T> bool sphereCone(const Vector3<T>& sphereCenter, T sphereRadius, const Vector3<T>& coneOrigin, const Vector3<T>& coneNormal, T sinAngle, T tanAngleSqPlusOne);
 
@@ -392,9 +382,7 @@ cone's axis and are tested for intersection with the cone using
 
 The @p tanAngleSqPlusOne parameter can be precomputed like this:
 
-@code{.cpp}
-T tanAngleSqPlusOne = Math::pow<2>(Math::tan(angle*T(0.5))) + T(1);
-@endcode
+@snippet MagnumMath.cpp Intersection-tanAngleSqPlusOne
 */
 template<class T> bool aabbCone(const Vector3<T>& aabbCenter, const Vector3<T>& aabbExtents, const Vector3<T>& coneOrigin, const Vector3<T>& coneNormal, T tanAngleSqPlusOne);
 
@@ -422,16 +410,15 @@ template<class T> bool rangeCone(const Range3D<T>& range, const Vector3<T>& cone
     otherwise
 
 Converts the range into center/extents representation and passes it on to
-@ref aabbCone(const Vector3<T>&, const Vector3<T>&, const Vector3<T>&, const Vector3<T>&, T) "aabbCone()". The @p tanAngleSqPlusOne parameter can be precomputed like this:
+@ref aabbCone(const Vector3<T>&, const Vector3<T>&, const Vector3<T>&, const Vector3<T>&, T) "aabbCone()".
+The @p tanAngleSqPlusOne parameter can be precomputed like this:
 
-@code{.cpp}
-T tanAngleSqPlusOne = Math::pow<2>(Math::tan(angle*T(0.5))) + T(1);
-@endcode
+@snippet MagnumMath.cpp Intersection-tanAngleSqPlusOne
 */
 template<class T> bool rangeCone(const Range3D<T>& range, const Vector3<T>& coneOrigin, const Vector3<T>& coneNormal, const T tanAngleSqPlusOne);
 
 template<class T> bool pointFrustum(const Vector3<T>& point, const Frustum<T>& frustum) {
-    for(const Vector4<T>& plane: frustum.planes()) {
+    for(const Vector4<T>& plane: frustum) {
         /* The point is in front of one of the frustum planes (normals point
            outwards) */
         if(Distance::pointPlaneScaled<T>(point, plane) < T(0))
@@ -447,7 +434,7 @@ template<class T> bool rangeFrustum(const Range3D<T>& range, const Frustum<T>& f
     const Vector3<T> center = range.min() + range.max();
     const Vector3<T> extent = range.max() - range.min();
 
-    for(const Vector4<T>& plane: frustum.planes()) {
+    for(const Vector4<T>& plane: frustum) {
         const Vector3<T> absPlaneNormal = Math::abs(plane.xyz());
 
         const Float d = Math::dot(center, plane.xyz());
@@ -459,7 +446,7 @@ template<class T> bool rangeFrustum(const Range3D<T>& range, const Frustum<T>& f
 }
 
 template<class T> bool aabbFrustum(const Vector3<T>& aabbCenter, const Vector3<T>& aabbExtents, const Frustum<T>& frustum) {
-    for(const Vector4<T>& plane: frustum.planes()) {
+    for(const Vector4<T>& plane: frustum) {
         const Vector3<T> absPlaneNormal = Math::abs(plane.xyz());
 
         const Float d = Math::dot(aabbCenter, plane.xyz());
@@ -473,7 +460,7 @@ template<class T> bool aabbFrustum(const Vector3<T>& aabbCenter, const Vector3<T
 template<class T> bool sphereFrustum(const Vector3<T>& sphereCenter, const T sphereRadius, const Frustum<T>& frustum) {
     const T radiusSq = sphereRadius*sphereRadius;
 
-    for(const Vector4<T>& plane: frustum.planes()) {
+    for(const Vector4<T>& plane: frustum) {
         /* The sphere is in front of one of the frustum planes (normals point
            outwards) */
         if(Distance::pointPlaneScaled<T>(sphereCenter, plane) < -radiusSq)
