@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -30,7 +30,6 @@
  */
 
 #include <cstddef>
-#include <Corrade/configure.h>
 
 #include "Magnum/Types.h"
 
@@ -39,15 +38,9 @@ namespace Magnum { namespace Math {
 /** @todo Denormals to zero */
 
 #ifndef DOXYGEN_GENERATING_OUTPUT
-/* Class Constants used only statically */
+template<std::size_t> class BoolVector;
 
-template<UnsignedInt, UnsignedInt, class> class Bezier;
-template<UnsignedInt dimensions, class T> using QuadraticBezier = Bezier<2, dimensions, T>;
-template<UnsignedInt dimensions, class T> using CubicBezier = Bezier<3, dimensions, T>;
-template<class T> using QuadraticBezier2D = QuadraticBezier<2, T>;
-template<class T> using QuadraticBezier3D = QuadraticBezier<3, T>;
-template<class T> using CubicBezier2D = CubicBezier<2, T>;
-template<class T> using CubicBezier3D = CubicBezier<3, T>;
+template<class> struct Constants;
 
 template<class> class Complex;
 template<class> class Dual;
@@ -85,13 +78,33 @@ template<class> class Vector2;
 template<class> class Vector3;
 template<class> class Vector4;
 
+template<class> struct ColorHsv;
 template<class> class Color3;
 template<class> class Color4;
 
+template<UnsignedInt, UnsignedInt, class> class Bezier;
+template<UnsignedInt dimensions, class T> using QuadraticBezier = Bezier<2, dimensions, T>;
+template<UnsignedInt dimensions, class T> using CubicBezier = Bezier<3, dimensions, T>;
+template<class T> using QuadraticBezier2D = QuadraticBezier<2, T>;
+template<class T> using QuadraticBezier3D = QuadraticBezier<3, T>;
+template<class T> using CubicBezier2D = CubicBezier<2, T>;
+template<class T> using CubicBezier3D = CubicBezier<3, T>;
+
+template<class> class CubicHermite;
+template<class T> using CubicHermite1D = CubicHermite<T>;
+template<class T> using CubicHermite2D = CubicHermite<Vector2<T>>;
+template<class T> using CubicHermite3D = CubicHermite<Vector3<T>>;
+template<class T> using CubicHermiteComplex = CubicHermite<Complex<T>>;
+template<class T> using CubicHermiteQuaternion = CubicHermite<Quaternion<T>>;
+
 template<UnsignedInt, class> class Range;
 template<class T> using Range1D = Range<1, T>;
 template<class> class Range2D;
 template<class> class Range3D;
+
+namespace Implementation {
+    template<class> struct StrictWeakOrdering;
+}
 #endif
 
 }}
