@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -65,55 +65,37 @@ template<std::size_t size, class T> class Matrix: public RectangularMatrix<size,
         /**
          * @brief Default constructor
          *
-         * Creates identity matrix. @p value allows you to specify value on
-         * diagonal.
+         * Equivalent to @ref Matrix(IdentityInitT, T).
          */
-        constexpr /*implicit*/ Matrix(IdentityInitT = IdentityInit, T value = T(1)) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : RectangularMatrix<size, size, T>{typename Implementation::GenerateSequence<size>::Type(), Vector<size, T>(value)}
-            #endif
-            {}
-
-        /** @copydoc RectangularMatrix::RectangularMatrix(ZeroInitT) */
-        constexpr explicit Matrix(ZeroInitT) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : RectangularMatrix<size, size, T>{ZeroInit}
-            #endif
-            {}
-
-        /** @copydoc RectangularMatrix::RectangularMatrix(NoInitT) */
-        constexpr explicit Matrix(NoInitT) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : RectangularMatrix<size, size, T>{NoInit}
-            #endif
-            {}
-
-        /** @brief Construct matrix from column vectors */
-        template<class ...U> constexpr /*implicit*/ Matrix(const Vector<size, T>& first, const U&... next) noexcept: RectangularMatrix<size, size, T>(first, next...) {}
-
-        /** @brief Construct matrix with one value for all elements */
-        constexpr explicit Matrix(T value) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : RectangularMatrix<size, size, T>{typename Implementation::GenerateSequence<size>::Type(), value}
-            #endif
-            {}
+        constexpr /*implicit*/ Matrix() noexcept: RectangularMatrix<size, size, T>{typename Implementation::GenerateSequence<size>::Type(), Vector<size, T>(T(1))} {}
 
         /**
-         * @brief Construct matrix from another of different type
+         * @brief Construct an identity matrix
+         *
+         * The @p value allows you to specify a value on diagonal.
+         * @see @ref fromDiagonal()
+         */
+        constexpr explicit Matrix(IdentityInitT, T value = T(1)) noexcept: RectangularMatrix<size, size, T>{typename Implementation::GenerateSequence<size>::Type(), Vector<size, T>(value)} {}
+
+        /** @copydoc RectangularMatrix::RectangularMatrix(ZeroInitT) */
+        constexpr explicit Matrix(ZeroInitT) noexcept: RectangularMatrix<size, size, T>{ZeroInit} {}
+
+        /** @copydoc RectangularMatrix::RectangularMatrix(NoInitT) */
+        constexpr explicit Matrix(NoInitT) noexcept: RectangularMatrix<size, size, T>{NoInit} {}
+
+        /** @brief Construct from column vectors */
+        template<class ...U> constexpr /*implicit*/ Matrix(const Vector<size, T>& first, const U&... next) noexcept: RectangularMatrix<size, size, T>(first, next...) {}
+
+        /** @brief Construct with one value for all elements */
+        constexpr explicit Matrix(T value) noexcept: RectangularMatrix<size, size, T>{typename Implementation::GenerateSequence<size>::Type(), value} {}
+
+        /**
+         * @brief Construct from a matrix of adifferent type
          *
          * Performs only default casting on the values, no rounding or
          * anything else. Example usage:
          *
-         * @code{.cpp}
-         * Matrix2x2<Float> floatingPoint({1.3f, 2.7f},
-         *                                {-15.0f, 7.0f});
-         * Matrix2x2<Byte> integral(floatingPoint);
-         * // integral == {{1, 2}, {-15, 7}}
-         * @endcode
+         * @snippet MagnumMath.cpp Matrix-conversion
          */
         template<class U> constexpr explicit Matrix(const RectangularMatrix<size, size, U>& other) noexcept: RectangularMatrix<size, size, T>(other) {}
 
@@ -121,18 +103,13 @@ template<std::size_t size, class T> class Matrix: public RectangularMatrix<size,
         template<class U, class V = decltype(Implementation::RectangularMatrixConverter<size, size, T, U>::from(std::declval<U>()))> constexpr explicit Matrix(const U& other): RectangularMatrix<size, size, T>(Implementation::RectangularMatrixConverter<size, size, T, U>::from(other)) {}
 
         /**
-         * @brief Construct matrix by slicing or expanding another of a different size
+         * @brief Construct by slicing or expanding a matrix of different size
          *
          * If the other matrix is larger, takes only the first @cpp size @ce
          * columns and rows from it; if the other matrix is smaller, it's
          * expanded to an identity (ones on diagonal, zeros elsewhere).
          */
-        template<std::size_t otherSize> constexpr explicit Matrix(const RectangularMatrix<otherSize, otherSize, T>& other) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : Matrix<size, T>{typename Implementation::GenerateSequence<size>::Type(), other}
-            #endif
-            {}
+        template<std::size_t otherSize> constexpr explicit Matrix(const RectangularMatrix<otherSize, otherSize, T>& other) noexcept: Matrix<size, T>{typename Implementation::GenerateSequence<size>::Type(), other} {}
 
         /** @brief Copy constructor */
         constexpr /*implicit*/ Matrix(const RectangularMatrix<size, size, T>& other) noexcept: RectangularMatrix<size, size, T>(other) {}
@@ -158,33 +135,119 @@ template<std::size_t size, class T> class Matrix: public RectangularMatrix<size,
          */
         T trace() const { return RectangularMatrix<size, size, T>::diagonal().sum(); }
 
-        /** @brief Matrix without given column and row */
+        /**
+         * @brief Matrix without given column and row
+         *
+         * For the following matrix @f$ \boldsymbol{M} @f$,
+         * @f$ \boldsymbol{M}_{3,2} @f$ is defined as: @f[
+         *  \begin{array}{rcl}
+         *      \boldsymbol{M} & = & \begin{pmatrix}
+         *          \,\,\,1 & 4 & 7 \\
+         *          \,\,\,3 & 0 & 5 \\
+         *          -1 & 9 & \!11 \\
+         *      \end{pmatrix} \\[2em]
+         *      \boldsymbol{M}_{2,3} & = & \begin{pmatrix}
+         *          \,\,1 & 4 & \Box\, \\
+         *          \,\Box & \Box & \Box\, \\
+         *          -1 & 9 & \Box\, \\
+         *      \end{pmatrix} = \begin{pmatrix}
+         *          \,\,\,1 & 4\, \\
+         *          -1 & 9\, \\
+         *      \end{pmatrix}
+         *  \end{array}
+         * @f]
+         *
+         * @see @ref cofactor(), @ref adjugate(), @ref determinant()
+         */
         Matrix<size-1, T> ij(std::size_t skipCol, std::size_t skipRow) const;
 
+        /**
+         * @brief Cofactor
+         *
+         * Cofactor @f$ C_{i,j} @f$ of a matrix @f$ \boldsymbol{M} @f$ is
+         * defined as @f$ C_{i,j} = (-1)^{i + j} \det \boldsymbol{M}_{i,j} @f$,
+         * with @f$ \boldsymbol{M}_{i,j} @f$ being @f$ \boldsymbol{M} @f$
+         * without the i-th column and j-th row. For example, calculating
+         * @f$ C_{3,2} @f$ of @f$ \boldsymbol{M} @f$ defined as @f[
+         *  \boldsymbol{M} = \begin{pmatrix}
+         *      \,\,\,1 & 4 & 7 \\
+         *      \,\,\,3 & 0 & 5 \\
+         *      -1 & 9 & \!11 \\
+         *  \end{pmatrix}
+         * @f]
+         *
+         * @m_class{m-noindent}
+         *
+         * would be @f[
+         *  C_{3,2} = (-1)^{2 + 3} \det \begin{pmatrix}
+         *      \,\,1 & 4 & \Box\, \\
+         *      \,\Box & \Box & \Box\, \\
+         *      -1 & 9 & \Box\, \\
+         *  \end{pmatrix} = -\det \begin{pmatrix}
+         *      \,\,\,1 & 4\, \\
+         *      -1 & 9\, \\
+         *  \end{pmatrix} = -(9-(-4)) = -13
+         * @f]
+         *
+         * @see @ref ij(), @ref comatrix(), @ref adjugate()
+         */
+        T cofactor(std::size_t col, std::size_t row) const;
+
+        /**
+         * @brief Matrix of cofactors
+         *
+         * A cofactor matrix @f$ \boldsymbol{C} @f$ of a matrix
+         * @f$ \boldsymbol{M} @f$ is defined as the following, with each
+         * @f$ C_{i,j} @f$ calculated as in @ref cofactor(). @f[
+         *  \boldsymbol C = \begin{pmatrix}
+         *      C_{1,1}  & C_{2,1} & \cdots &   C_{n,1} \\
+         *      C_{1,2}  & C_{2,2} & \cdots &   C_{n,2} \\
+         *      \vdots & \vdots & \ddots & \vdots \\
+         *      C_{1,n}  & C_{2,n} & \cdots &  C_{n,n}
+         *  \end{pmatrix}
+         * @f]
+         *
+         * @see @ref Matrix4::normalMatrix(), @ref ij(), @ref adjugate()
+         */
+        Matrix<size, T> comatrix() const;
+
+        /**
+         * @brief Adjugate matrix
+         *
+         * @f$ adj(A) @f$. Transpose of a @ref comatrix(), used for example to
+         * calculate an @ref inverted() matrix.
+         */
+        Matrix<size, T> adjugate() const;
+
         /**
          * @brief Determinant
          *
-         * Returns `0` if the matrix is noninvertible and `1` if the matrix is
-         * orthogonal. Computed recursively using Laplace's formula: @f[
-         *      \det(A) = \sum_{j=1}^n (-1)^{i+j} a_{i,j} \det(A^{i,j})
-         * @f] @f$ A^{i, j} @f$ is matrix without i-th row and j-th column, see
-         * @ref ij(). The formula is expanded down to 2x2 matrix, where the
-         * determinant is computed directly: @f[
-         *      \det(A) = a_{0, 0} a_{1, 1} - a_{1, 0} a_{0, 1}
+         * Returns 0 if the matrix is noninvertible and 1 if the matrix is
+         * orthogonal. Computed recursively using
+         * <a href="https://en.wikipedia.org/wiki/Determinant#Laplace's_formula_and_the_adjugate_matrix">Laplace's formula</a>: @f[
+         *      \det \boldsymbol{A} = \sum_{j=1}^n (-1)^{i+j} a_{i,j} \det \boldsymbol{A}_{i,j}
+         * @f] @f$ \boldsymbol{A}_{i,j} @f$ is @f$ \boldsymbol{A} @f$ without
+         * the i-th column and j-th row. The formula is recursed down to a 2x2
+         * matrix, where the determinant is calculated directly: @f[
+         *      \det \boldsymbol{A} = a_{0, 0} a_{1, 1} - a_{1, 0} a_{0, 1}
          * @f]
+         *
+         * @see @ref ij()
          */
         T determinant() const { return Implementation::MatrixDeterminant<size, T>()(*this); }
 
         /**
          * @brief Inverted matrix
          *
-         * Computed using Cramer's rule: @f[
-         *      A^{-1} = \frac{1}{\det(A)} Adj(A)
+         * Calculated using <a href="https://en.wikipedia.org/wiki/Cramer's_rule">Cramer's rule</a> and @ref adjugate(), or equivalently
+         * using a @ref comatrix(): @f[
+         *      \boldsymbol{A}^{-1} = \frac{1}{\det \boldsymbol{A}} adj(\boldsymbol{A}) = \frac{1}{\det \boldsymbol{A}} \boldsymbol{C}^T
          * @f]
          * See @ref invertedOrthogonal(), @ref Matrix3::invertedRigid() and
          * @ref Matrix4::invertedRigid() which are faster alternatives for
          * particular matrix types.
-         * @see @ref Algorithms::gaussJordanInverted()
+         * @see @ref Algorithms::gaussJordanInverted(),
+         *      @ref Matrix4::normalMatrix()
          * @m_keyword{inverse(),GLSL inverse(),}
          */
         Matrix<size, T> inverted() const;
@@ -194,7 +257,7 @@ template<std::size_t size, class T> class Matrix: public RectangularMatrix<size,
          *
          * Equivalent to @ref transposed(), expects that the matrix is
          * orthogonal. @f[
-         *      A^{-1} = A^T
+         *      \boldsymbol{A}^{-1} = \boldsymbol{A}^T
          * @f]
          * @see @ref inverted(), @ref isOrthogonal(),
          *      @ref Matrix3::invertedRigid(),
@@ -224,6 +287,8 @@ template<std::size_t size, class T> class Matrix: public RectangularMatrix<size,
         #endif
 
     private:
+        friend struct Implementation::MatrixDeterminant<size, T>;
+
         /* Implementation for RectangularMatrix<cols, rows, T>::RectangularMatrix(const RectangularMatrix<cols, rows, U>&) */
         template<std::size_t otherSize, std::size_t ...col> constexpr explicit Matrix(Implementation::Sequence<col...>, const RectangularMatrix<otherSize, otherSize, T>& other) noexcept: RectangularMatrix<size, size, T>{Implementation::valueOrIdentityVector<size, col>(other)...} {}
 };
@@ -297,42 +362,98 @@ MAGNUM_MATRIX_OPERATOR_IMPLEMENTATION(Matrix<size, T>)
 namespace Implementation {
 
 template<std::size_t size, class T> struct MatrixDeterminant {
-    T operator()(const Matrix<size, T>& m);
+    T operator()(const Matrix<size, T>& m) {
+        T out(0);
+
+        /* Using ._data[] instead of [] to avoid function call indirection on
+           debug builds (saves a lot, yet doesn't obfuscate too much) */
+        for(std::size_t col = 0; col != size; ++col)
+            out += m._data[col]._data[0]*m.cofactor(col, 0);
+
+        return out;
+    }
+
+    T operator()(const Matrix<size + 1, T>& m, const std::size_t skipCol, const std::size_t skipRow) {
+        return m.ij(skipCol, skipRow).determinant();
+    }
 };
 
-template<std::size_t size, class T> T MatrixDeterminant<size, T>::operator()(const Matrix<size, T>& m) {
-    T out(0);
+/* This is not *critically* needed here (the specializations for 2x2 and 1x1
+   are technically enough to make things work), but together with the raw data
+   access it speeds up the debug build five times, so I think it's worth to
+   have it */
+template<class T> struct MatrixDeterminant<3, T> {
+    constexpr T operator()(const Matrix<3, T>& m) const {
+        /* Using ._data[] instead of [] to avoid function call indirection
+           on debug builds (saves a lot, yet doesn't obfuscate too much) */
+        return m._data[0]._data[0]*((m._data[1]._data[1]*m._data[2]._data[2]) - (m._data[2]._data[1]*m._data[1]._data[2])) -
+            m._data[0]._data[1]*(m._data[1]._data[0]*m._data[2]._data[2] - m._data[2]._data[0]*m._data[1]._data[2]) +
+            m._data[0]._data[2]*(m._data[1]._data[0]*m._data[2]._data[1] - m._data[2]._data[0]*m._data[1]._data[1]);
+    }
 
-    for(std::size_t col = 0; col != size; ++col)
-        out += ((col & 1) ? -1 : 1)*m[col][0]*m.ij(col, 0).determinant();
-
-    return out;
-}
+    /* Used internally by cofactor(), basically just an inlined variant of
+       ij(skipCol, skipRow).determinant() */
+    constexpr T operator()(const Matrix<4, T>& m, const std::size_t skipCol, const std::size_t skipRow) const {
+        #define _col(i) _data[i + (i >= skipCol)]
+        #define _row(i) _data[i + (i >= skipRow)]
+        return
+            m._col(0)._row(0)*((m._col(1)._row(1)*m._col(2)._row(2)) - (m._col(2)._row(1)*m._col(1)._row(2))) -
+            m._col(0)._row(1)*(m._col(1)._row(0)*m._col(2)._row(2) - m._col(2)._row(0)*m._col(1)._row(2)) +
+            m._col(0)._row(2)*(m._col(1)._row(0)*m._col(2)._row(1) - m._col(2)._row(0)*m._col(1)._row(1));
+        #undef _col
+        #undef _row
+    }
+};
 
 template<class T> struct MatrixDeterminant<2, T> {
     constexpr T operator()(const Matrix<2, T>& m) const {
-        return m[0][0]*m[1][1] - m[1][0]*m[0][1];
+        /* Using ._data[] instead of [] to avoid function call indirection
+           on debug builds (saves a lot, yet doesn't obfuscate too much) */
+        return m._data[0]._data[0]*m._data[1]._data[1] - m._data[1]._data[0]*m._data[0]._data[1];
+    }
+
+    /* Used internally by cofactor(), basically just an inlined variant of
+       ij(skipCol, skipRow).determinant() */
+    constexpr T operator()(const Matrix<3, T>& m, const std::size_t skipCol, const std::size_t skipRow) const {
+        #define _col(i) _data[i + (i >= skipCol)]
+        #define _row(i) _data[i + (i >= skipRow)]
+        return m._col(0)._row(0)*m._col(1)._row(1) - m._col(1)._row(0)*m._col(0)._row(1);
+        #undef _col
+        #undef _row
     }
 };
 
 template<class T> struct MatrixDeterminant<1, T> {
     constexpr T operator()(const Matrix<1, T>& m) const {
-        return m[0][0];
+        /* Using ._data[] instead of [] to avoid function call indirection
+           on debug builds (saves a lot, yet doesn't obfuscate too much) */
+        return m._data[0]._data[0];
+    }
+
+    /* Used internally by cofactor(), basically just an inlined variant of
+       ij(skipCol, skipRow).determinant() */
+    constexpr T operator()(const Matrix<2, T>& m, const std::size_t skipCol, const std::size_t skipRow) const {
+        return m._data[0 + (0 >= skipCol)]._data[0 + (0 >= skipRow)];
     }
 };
 
+template<std::size_t size, class T> struct StrictWeakOrdering<Matrix<size, T>>: StrictWeakOrdering<RectangularMatrix<size, size, T>> {};
+
 }
 #endif
 
 template<std::size_t size, class T> bool Matrix<size, T>::isOrthogonal() const {
+    /* Using ._data[] instead of [] to avoid function call indirection on debug
+       builds (saves a lot, yet doesn't obfuscate too much) */
+
     /* Normality */
     for(std::size_t i = 0; i != size; ++i)
-        if(!(*this)[i].isNormalized()) return false;
+        if(!RectangularMatrix<size, size, T>::_data[i].isNormalized()) return false;
 
     /* Orthogonality */
     for(std::size_t i = 0; i != size-1; ++i)
         for(std::size_t j = i+1; j != size; ++j)
-            if(dot((*this)[i], (*this)[j]) > TypeTraits<T>::epsilon())
+            if(dot(RectangularMatrix<size, size, T>::_data[i], RectangularMatrix<size, size, T>::_data[j]) > TypeTraits<T>::epsilon())
                 return false;
 
     return true;
@@ -341,31 +462,50 @@ template<std::size_t size, class T> bool Matrix<size, T>::isOrthogonal() const {
 template<std::size_t size, class T> Matrix<size-1, T> Matrix<size, T>::ij(const std::size_t skipCol, const std::size_t skipRow) const {
     Matrix<size-1, T> out{NoInit};
 
+    /* Using ._data[] instead of [] to avoid function call indirection on debug
+       builds (saves a lot, yet doesn't obfuscate too much) */
     for(std::size_t col = 0; col != size-1; ++col)
         for(std::size_t row = 0; row != size-1; ++row)
-            out[col][row] = (*this)[col + (col >= skipCol)]
-                                    [row + (row >= skipRow)];
+            out._data[col]._data[row] = RectangularMatrix<size, size, T>::
+                _data[col + (col >= skipCol)]
+               ._data[row + (row >= skipRow)];
+
+    return out;
+}
+
+template<std::size_t size, class T> T Matrix<size, T>::cofactor(std::size_t col, std::size_t row) const {
+    return (((row+col) & 1) ? -1 : 1)*Implementation::MatrixDeterminant<size - 1, T>()(*this, col, row);
+}
+
+template<std::size_t size, class T> Matrix<size, T> Matrix<size, T>::comatrix() const {
+    Matrix<size, T> out{NoInit};
+
+    /* Using ._data[] instead of [] to avoid function call indirection on debug
+       builds (saves a lot, yet doesn't obfuscate too much) */
+    for(std::size_t col = 0; col != size; ++col)
+        for(std::size_t row = 0; row != size; ++row)
+            out._data[col]._data[row] = cofactor(col, row);
+
+    return out;
+}
+
+template<std::size_t size, class T> Matrix<size, T> Matrix<size, T>::adjugate() const {
+    Matrix<size, T> out{NoInit};
+
+    /* Same as comatrix(), except using cofactor(row, col) instead of
+       cofactor(col, row). Could also be just comatrix().transpose() but since
+       this is used in inverted(), each extra operation hurts. */
+    for(std::size_t col = 0; col != size; ++col)
+        for(std::size_t row = 0; row != size; ++row)
+            out._data[col]._data[row] = cofactor(row, col);
 
     return out;
 }
 
 template<std::size_t size, class T> Matrix<size, T> Matrix<size, T>::inverted() const {
-    Matrix<size, T> out{NoInit};
-
-    const T _determinant = determinant();
-
-    for(std::size_t col = 0; col != size; ++col)
-        for(std::size_t row = 0; row != size; ++row)
-            out[col][row] = (((row+col) & 1) ? -1 : 1)*ij(row, col).determinant()/_determinant;
-
-    return out;
+    return adjugate()/determinant();
 }
 
 }}
 
-namespace Corrade { namespace Utility {
-    /** @configurationvalue{Magnum::Math::Matrix} */
-    template<std::size_t size, class T> struct ConfigurationValue<Magnum::Math::Matrix<size, T>>: public ConfigurationValue<Magnum::Math::RectangularMatrix<size, size, T>> {};
-}}
-
 #endif
