@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -120,7 +120,7 @@ template<class T> class Matrix4: public Matrix4x4<T> {
         static Matrix4<T> rotation(Rad<T> angle, const Vector3<T>& normalizedAxis);
 
         /**
-         * @brief 3D rotation matrix around X axis
+         * @brief 3D rotation matrix around the X axis
          * @param angle Rotation angle (counterclockwise)
          *
          * Faster than calling @cpp Matrix4::rotation(angle, Vector3::xAxis()) @ce. @f[
@@ -138,7 +138,7 @@ template<class T> class Matrix4: public Matrix4x4<T> {
         static Matrix4<T> rotationX(Rad<T> angle);
 
         /**
-         * @brief 3D rotation matrix around Y axis
+         * @brief 3D rotation matrix around the Y axis
          * @param angle Rotation angle (counterclockwise)
          *
          * Faster than calling @cpp Matrix4::rotation(angle, Vector3::yAxis()) @ce. @f[
@@ -156,7 +156,7 @@ template<class T> class Matrix4: public Matrix4x4<T> {
         static Matrix4<T> rotationY(Rad<T> angle);
 
         /**
-         * @brief 3D rotation matrix around Z axis
+         * @brief 3D rotation matrix around the Z axis
          * @param angle Rotation angle (counterclockwise)
          *
          * Faster than calling @cpp Matrix4::rotation(angle, Vector3::zAxis()) @ce. @f[
@@ -188,9 +188,9 @@ template<class T> class Matrix4: public Matrix4x4<T> {
         static Matrix4<T> reflection(const Vector3<T>& normal);
 
         /**
-         * @brief 3D shearing matrix along XY plane
-         * @param amountX   Amount of shearing along X axis
-         * @param amountY   Amount of shearing along Y axis
+         * @brief 3D shearing matrix along the XY plane
+         * @param amountX   Amount of shearing along the X axis
+         * @param amountY   Amount of shearing along the Y axis
          *
          * Z axis remains unchanged. @f[
          *      \boldsymbol{A} = \begin{pmatrix}
@@ -211,9 +211,9 @@ template<class T> class Matrix4: public Matrix4x4<T> {
         }
 
         /**
-         * @brief 3D shearing matrix along XZ plane
-         * @param amountX   Amount of shearing along X axis
-         * @param amountZ   Amount of shearing along Z axis
+         * @brief 3D shearing matrix along the XZ plane
+         * @param amountX   Amount of shearing along the X axis
+         * @param amountZ   Amount of shearing along the Z axis
          *
          * Y axis remains unchanged. @f[
          *      \boldsymbol{A} = \begin{pmatrix}
@@ -234,9 +234,9 @@ template<class T> class Matrix4: public Matrix4x4<T> {
         }
 
         /**
-         * @brief 3D shearing matrix along YZ plane
-         * @param amountY   Amount of shearing along Y axis
-         * @param amountZ   Amount of shearing along Z axis
+         * @brief 3D shearing matrix along the YZ plane
+         * @param amountY   Amount of shearing along the Y axis
+         * @param amountZ   Amount of shearing along the Z axis
          *
          * X axis remains unchanged. @f[
          *      \boldsymbol{A} = \begin{pmatrix}
@@ -298,10 +298,14 @@ template<class T> class Matrix4: public Matrix4x4<T> {
          *          0 & 0 & -1 & 0
          *      \end{pmatrix}
          * @f]
+         *
+         * If you need an off-center projection, use
+         * @ref perspectiveProjection(const Vector2<T>&, const Vector2<T>&, T, T)
+         * instead.
+         *
          * @see @ref perspectiveProjection(Rad<T> fov, T, T, T),
          *      @ref orthographicProjection(), @ref Matrix3::projection(),
          *      @ref Constants::inf()
-         * @m_keywords{gluPerspective()}
          */
         static Matrix4<T> perspectiveProjection(const Vector2<T>& size, T near, T far);
 
@@ -340,6 +344,12 @@ template<class T> class Matrix4: public Matrix4x4<T> {
          *      \end{pmatrix}
          * @f]
          *
+         * This function is similar to the classic @cpp gluPerspective() @ce,
+         * with the difference that @p fov is *horizontal* instead of vertical.
+         * If you need an off-center projection (as with the classic
+         * @cpp glFrustum() @ce function), use
+         * @ref perspectiveProjection(const Vector2<T>&, const Vector2<T>&, T, T).
+         *
          * @see @ref orthographicProjection(), @ref Matrix3::projection(),
          *      @ref Constants::inf()
          * @m_keywords{gluPerspective()}
@@ -348,6 +358,44 @@ template<class T> class Matrix4: public Matrix4x4<T> {
             return perspectiveProjection(T(2)*near*std::tan(T(fov)*T(0.5))*Vector2<T>::yScale(T(1)/aspectRatio), near, far);
         }
 
+        /**
+         * @brief 3D off-center perspective projection matrix
+         * @param bottomLeft    Bottom left corner of the near clipping plane
+         * @param topRight      Top right corner of the near clipping plane
+         * @param near          Distance to near clipping plane, positive is
+         *      ahead
+         * @param far           Distance to far clipping plane, positive is
+         *      ahead
+         *
+         * If @p far is finite, the result is: @f[
+         *      \boldsymbol{A} = \begin{pmatrix}
+         *          \frac{2n}{r - l} & 0 & \frac{r + l}{r - l} & 0 \\
+         *          0 & \frac{2n}{t - b} & \frac{t + b}{t - b} & 0 \\
+         *          0 & 0 & \frac{n + f}{n - f} & \frac{2nf}{n - f} \\
+         *          0 & 0 & -1 & 0
+         *      \end{pmatrix}
+         * @f]
+         *
+         * For infinite @p far, the result is: @f[
+         *      \boldsymbol{A} = \begin{pmatrix}
+         *          \frac{2n}{r - l} & 0 & \frac{r + l}{r - l} & 0 \\
+         *          0 & \frac{2n}{t - b} & \frac{t + b}{t - b} & 0 \\
+         *          0 & 0 & -1 & -2n \\
+         *          0 & 0 & -1 & 0
+         *      \end{pmatrix}
+         * @f]
+         *
+         * Equivalent to the classic @cpp glFrustum() @ce function. If
+         * @p bottomLeft and @p topRight are a negation of each other, this
+         * function is equivalent to @ref perspectiveProjection(const Vector2<T>&, T, T).
+         *
+         * @see @ref perspectiveProjection(Rad<T> fov, T, T, T),
+         *      @ref orthographicProjection(), @ref Matrix3::projection(),
+         *      @ref Constants::inf()
+         * @m_keywords{glFrustum()}
+         */
+        static Matrix4<T> perspectiveProjection(const Vector2<T>& bottomLeft, const Vector2<T>& topRight, T near, T far);
+
         /**
          * @brief Matrix oriented towards a specific point
          * @param eye       Location to place the matrix
@@ -365,7 +413,7 @@ template<class T> class Matrix4: public Matrix4x4<T> {
         static Matrix4<T> lookAt(const Vector3<T>& eye, const Vector3<T>& target, const Vector3<T>& up);
 
         /**
-         * @brief Create matrix from rotation/scaling part and translation part
+         * @brief Create a matrix from a rotation/scaling part and a translation part
          * @param rotationScaling   Rotation/scaling part (upper-left 3x3
          *      matrix)
          * @param translation       Translation part (first three elements of
@@ -383,51 +431,37 @@ template<class T> class Matrix4: public Matrix4x4<T> {
         /**
          * @brief Default constructor
          *
-         * Creates identity matrix. @p value allows you to specify value on
-         * diagonal.
+         * Equivalent to @ref Matrix4(IdentityInitT, T).
          */
-        constexpr /*implicit*/ Matrix4(IdentityInitT = IdentityInit, T value = T{1}) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : Matrix4x4<T>{IdentityInit, value}
-            #endif
-            {}
+        constexpr /*implicit*/ Matrix4() noexcept: Matrix4x4<T>{IdentityInit, T(1)} {}
+
+        /**
+         * @brief Construct an identity matrix
+         *
+         * The @p value allows you to specify value on diagonal.
+         */
+        constexpr explicit Matrix4(IdentityInitT, T value = T{1}) noexcept: Matrix4x4<T>{IdentityInit, value} {}
 
         /** @copydoc Matrix::Matrix(ZeroInitT) */
-        constexpr explicit Matrix4(ZeroInitT) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : Matrix4x4<T>{ZeroInit}
-            #endif
-            {}
+        constexpr explicit Matrix4(ZeroInitT) noexcept: Matrix4x4<T>{ZeroInit} {}
 
         /** @copydoc Matrix::Matrix(NoInitT) */
-        constexpr explicit Matrix4(NoInitT) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : Matrix4x4<T>{NoInit}
-            #endif
-            {}
+        constexpr explicit Matrix4(NoInitT) noexcept: Matrix4x4<T>{NoInit} {}
 
-        /** @brief Construct matrix from column vectors */
+        /** @brief Construct from column vectors */
         constexpr /*implicit*/ Matrix4(const Vector4<T>& first, const Vector4<T>& second, const Vector4<T>& third, const Vector4<T>& fourth) noexcept: Matrix4x4<T>(first, second, third, fourth) {}
 
-        /** @brief Construct matrix with one value for all elements */
-        constexpr explicit Matrix4(T value) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : Matrix4x4<T>{value}
-            #endif
-            {}
+        /** @brief Construct with one value for all elements */
+        constexpr explicit Matrix4(T value) noexcept: Matrix4x4<T>{value} {}
 
         /** @copydoc Matrix::Matrix(const RectangularMatrix<size, size, U>&) */
         template<class U> constexpr explicit Matrix4(const RectangularMatrix<4, 4, U>& other) noexcept: Matrix4x4<T>(other) {}
 
-        /** @brief Construct matrix from external representation */
+        /** @brief Construct from external representation */
         template<class U, class V = decltype(Implementation::RectangularMatrixConverter<4, 4, T, U>::from(std::declval<U>()))> constexpr explicit Matrix4(const U& other): Matrix4x4<T>(Implementation::RectangularMatrixConverter<4, 4, T, U>::from(other)) {}
 
         /**
-         * @brief Construct matrix by slicing or expanding another of a different size
+         * @brief Construct by slicing or expanding a matrix of a different size
          *
          * If the other matrix is larger, takes only the first @cpp size @ce
          * columns and rows from it; if the other matrix is smaller, it's
@@ -439,7 +473,7 @@ template<class T> class Matrix4: public Matrix4x4<T> {
         constexpr /*implicit*/ Matrix4(const RectangularMatrix<4, 4, T>& other) noexcept: Matrix4x4<T>(other) {}
 
         /**
-         * @brief Check whether the matrix represents rigid transformation
+         * @brief Check whether the matrix represents a rigid transformation
          *
          * Rigid transformation consists only of rotation and translation (i.e.
          * no scaling or projection).
@@ -480,7 +514,7 @@ template<class T> class Matrix4: public Matrix4x4<T> {
         }
 
         /**
-         * @brief 3D rotation and scaling part of the matrix
+         * @brief 3D rotation and shear part of the matrix
          *
          * Normalized upper-left 3x3 part of the matrix. Assuming the following
          * matrix, with the upper-left 3x3 part represented by column vectors
@@ -647,7 +681,7 @@ template<class T> class Matrix4: public Matrix4x4<T> {
         }
 
         /**
-         * @brief Non-uniform scaling part of the matrix, squared
+         * @brief Non-uniform scaling part of the matrix
          *
          * Length of vectors in upper-left 3x3 part of the matrix. Use the
          * faster alternative @ref scalingSquared() where possible. Assuming
@@ -763,6 +797,29 @@ template<class T> class Matrix4: public Matrix4x4<T> {
          */
         T uniformScaling() const { return std::sqrt(uniformScalingSquared()); }
 
+        /**
+         * @brief Normal matrix
+         *
+         * Returns @ref comatrix() of the upper-left 3x3 part of the matrix.
+         * Compared to the classic transformation @f$ (\boldsymbol{M}^{-1})^T @f$,
+         * which is done in order to preserve correct normal orientation for
+         * non-uniform scale and skew, this preserves it also when reflection
+         * is involved. Moreover it's also faster to calculate since we need
+         * just the @m_class{m-success} @f$ \boldsymbol{C} @f$ part of the
+         * inverse transpose: @f[
+         *  (\boldsymbol{M}^{-1})^T = \frac{1}{\det \boldsymbol{A}} \color{m-success} \boldsymbol{C}
+         * @f]
+         *
+         * Based on the [Normals Revisited](https://github.com/graphitemaster/normals_revisited)
+         * article by Dale Weiler.
+         * @see @ref inverted()
+         */
+        Matrix3x3<T> normalMatrix() const {
+            return Matrix3x3<T>{(*this)[0].xyz(),
+                                (*this)[1].xyz(),
+                                (*this)[2].xyz()}.comatrix();
+        }
+
         /**
          * @brief Right-pointing 3D vector
          *
@@ -851,9 +908,9 @@ template<class T> class Matrix4: public Matrix4x4<T> {
         Matrix4<T> invertedRigid() const;
 
         /**
-         * @brief Transform 3D vector with the matrix
+         * @brief Transform a 3D vector with the matrix
          *
-         * Unlike in @ref transformVector(), translation is not involved in the
+         * Unlike in @ref transformPoint(), translation is not involved in the
          * transformation. @f[
          *      \boldsymbol v' = \boldsymbol M \begin{pmatrix} v_x \\ v_y \\ v_z \\ 0 \end{pmatrix}
          * @f]
@@ -866,7 +923,7 @@ template<class T> class Matrix4: public Matrix4x4<T> {
         }
 
         /**
-         * @brief Transform 3D point with the matrix
+         * @brief Transform a 3D point with the matrix
          *
          * Unlike in @ref transformVector(), translation is also involved in
          * the transformation. @f[
@@ -969,18 +1026,41 @@ template<class T> Matrix4<T> Matrix4<T>::orthographicProjection(const Vector2<T>
 template<class T> Matrix4<T> Matrix4<T>::perspectiveProjection(const Vector2<T>& size, const T near, const T far) {
     const Vector2<T> xyScale = 2*near/size;
 
+    T m22, m32;
     if(far == Constants<T>::inf()) {
-        return {{xyScale.x(),        T(0),                 T(0),  T(0)},
-                {       T(0), xyScale.y(),                 T(0),  T(0)},
-                {       T(0),        T(0),                T(-1), T(-1)},
-                {       T(0),        T(0),           T(-2)*near,  T(0)}};
+        m22 = T(-1);
+        m32 = T(-2)*near;
     } else {
         const T zScale = T(1.0)/(near-far);
-        return {{xyScale.x(),        T(0),                 T(0),  T(0)},
-                {       T(0), xyScale.y(),                 T(0),  T(0)},
-                {       T(0),        T(0),    (far+near)*zScale, T(-1)},
-                {       T(0),        T(0), T(2)*far*near*zScale,  T(0)}};
+        m22 = (far+near)*zScale;
+        m32 = T(2)*far*near*zScale;
     }
+
+    return {{xyScale.x(),        T(0), T(0),  T(0)},
+            {       T(0), xyScale.y(), T(0),  T(0)},
+            {       T(0),        T(0), m22,  T(-1)},
+            {       T(0),        T(0), m32,  T(0)}};
+}
+
+template<class T> Matrix4<T> Matrix4<T>::perspectiveProjection(const Vector2<T>& bottomLeft, const Vector2<T>& topRight, const T near, const T far) {
+    const Vector2<T> xyDifference = topRight - bottomLeft;
+    const Vector2<T> xyScale = 2*near/xyDifference;
+    const Vector2<T> xyOffset = (topRight + bottomLeft)/xyDifference;
+
+    T m22, m32;
+    if(far == Constants<T>::inf()) {
+        m22 = T(-1);
+        m32 = T(-2)*near;
+    } else {
+        const T zScale = T(1.0)/(near-far);
+        m22 = (far+near)*zScale;
+        m32 = T(2)*far*near*zScale;
+    }
+
+    return {{ xyScale.x(),         T(0), T(0),  T(0)},
+            {        T(0),  xyScale.y(), T(0),  T(0)},
+            {xyOffset.x(), xyOffset.y(), m22,  T(-1)},
+            {        T(0),         T(0), m32,  T(0)}};
 }
 
 template<class T> Matrix4<T> Matrix4<T>::lookAt(const Vector3<T>& eye, const Vector3<T>& target, const Vector3<T>& up) {
@@ -1024,11 +1104,10 @@ template<class T> Matrix4<T> Matrix4<T>::invertedRigid() const {
     return from(inverseRotation, inverseRotation*-translation());
 }
 
-}}
+namespace Implementation {
+    template<class T> struct StrictWeakOrdering<Matrix4<T>>: StrictWeakOrdering<RectangularMatrix<4, 4, T>> {};
+}
 
-namespace Corrade { namespace Utility {
-    /** @configurationvalue{Magnum::Math::Matrix4} */
-    template<class T> struct ConfigurationValue<Magnum::Math::Matrix4<T>>: public ConfigurationValue<Magnum::Math::Matrix4x4<T>> {};
 }}
 
 #endif
