@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -30,8 +30,6 @@
  */
 
 #include "Magnum/GL/AbstractShaderProgram.h"
-#include "Magnum/Math/Color.h"
-#include "Magnum/Math/Matrix4.h"
 #include "Magnum/Shaders/Generic.h"
 #include "Magnum/Shaders/visibility.h"
 
@@ -40,19 +38,20 @@ namespace Magnum { namespace Shaders {
 /**
 @brief Phong shader
 
-Uses ambient, diffuse and specular color or texture. For colored mesh you need
-to provide the @ref Position and @ref Normal attributes in your triangle mesh.
-By default, the shader renders the mesh with a white color in an identity
+Uses ambient, diffuse and specular color or texture. For a colored mesh you
+need to provide the @ref Position and @ref Normal attributes in your triangle
+mesh. By default, the shader renders the mesh with a white color in an identity
 transformation. Use @ref setTransformationMatrix(), @ref setNormalMatrix(),
 @ref setProjectionMatrix(), @ref setLightPosition() and others to configure
 the shader.
 
-If you want to use textures, you need to provide also @ref TextureCoordinates
-attribute. Pass appropriate @ref Flags to constructor and then at render time
-don't forget to also call appropriate subset of @ref bindAmbientTexture(),
-@ref bindDiffuseTexture() and @ref bindSpecularTexture() (or the combined
-@ref bindTextures()). The texture is multipled by the color, which is by
-default set to fully opaque white for enabled textures.
+If you want to use textures, you need to provide also the
+@ref TextureCoordinates attribute. Pass appropriate @ref Flag combination to
+the constructor and then at render time don't forget to also call appropriate
+subset of @ref bindAmbientTexture(), @ref bindDiffuseTexture() and
+@ref bindSpecularTexture() (or the combined @ref bindTextures()). The texture
+is multipled by the color, which is by default set to fully opaque white for
+enabled textures.
 
 @image html shaders-phong.png width=256px
 
@@ -87,14 +86,35 @@ operation which is known to have considerable performance impact on some
 platforms. With proper depth sorting and blending you'll usually get much
 better performance and output quality.
 
-For general alpha-masked drawing you need to provide ambient texture with alpha
-channel and set alpha channel of diffuse/specular color to @cpp 0.0f @ce so
-only ambient alpha will be taken into account. If you have diffuse texture
+For general alpha-masked drawing you need to provide an ambient texture with
+alpha channel and set alpha channel of the diffuse/specular color to @cpp 0.0f @ce
+so only ambient alpha will be taken into account. If you have a diffuse texture
 combined with the alpha mask, you can use that texture for both ambient and
 diffuse part and then separate the alpha like this:
 
 @snippet MagnumShaders.cpp Phong-usage-alpha
 
+@subsection Shaders-Phong-usage-object-id Object ID output
+
+The shader supports writing object ID to the framebuffer for object picking or
+other annotation purposes. Enable it using @ref Flag::ObjectId and set up an
+integer buffer attached to the @ref ObjectIdOutput attachment. The
+functionality is practically the same as in the @ref Flat shader, see its
+@ref Shaders-Flat-usage-object-id documentation for more information and usage
+example.
+
+@requires_gles30 Object ID output requires integer buffer attachments, which
+    are not available in OpenGL ES 2.0 or WebGL 1.0.
+
+@section Shaders-Phong-zero-lights Zero lights
+
+Creating this shader with zero lights makes its output equivalent to the
+@ref Flat3D shader --- only @ref setAmbientColor() and @ref bindAmbientTexture()
+(if @ref Flag::AmbientTexture is enabled) are taken into account, which
+correspond to @ref Flat::setColor() and @ref Flat::bindTexture(). This is
+useful to reduce complexity in apps that render models with pre-baked lights.
+In addition, enabling @ref Flag::VertexColor and using a default ambient color with no texturing makes this shader equivalent to @ref VertexColor.
+
 @see @ref shaders
 */
 class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
@@ -115,6 +135,15 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
          */
         typedef Generic3D::Normal Normal;
 
+        /**
+         * @brief Tangent direction
+         *
+         * @ref shaders-generic "Generic attribute",
+         * @ref Magnum::Vector3 "Vector3", used only if
+         * @ref Flag::NormalTexture is set.
+         */
+        typedef Generic3D::Tangent Tangent;
+
         /**
          * @brief 2D texture coordinates
          *
@@ -125,6 +154,47 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
          */
         typedef Generic3D::TextureCoordinates TextureCoordinates;
 
+        /**
+         * @brief Three-component vertex color
+         *
+         * @ref shaders-generic "Generic attribute", @ref Magnum::Color3. Use
+         * either this or the @ref Color4 attribute. Used only if
+         * @ref Flag::VertexColor is set.
+         */
+        typedef Generic3D::Color3 Color3;
+
+        /**
+         * @brief Four-component vertex color
+         *
+         * @ref shaders-generic "Generic attribute", @ref Magnum::Color4. Use
+         * either this or the @ref Color3 attribute. Used only if
+         * @ref Flag::VertexColor is set.
+         */
+        typedef Generic3D::Color4 Color4;
+
+        enum: UnsignedInt {
+            /**
+             * Color shader output. @ref shaders-generic "Generic output",
+             * present always. Expects three- or four-component floating-point
+             * or normalized buffer attachment.
+             */
+            ColorOutput = Generic3D::ColorOutput,
+
+            #ifndef MAGNUM_TARGET_GLES2
+            /**
+             * Object ID shader output. @ref shaders-generic "Generic output",
+             * present only if @ref Flag::ObjectId is set. Expects a
+             * single-component unsigned integral attachment. Writes the value
+             * set in @ref setObjectId() there, see
+             * @ref Shaders-Phong-usage-object-id for more information.
+             * @requires_gles30 Object ID output requires integer buffer
+             *      attachments, which are not available in OpenGL ES 2.0 or
+             *      WebGL 1.0.
+             */
+            ObjectIdOutput = Generic3D::ObjectIdOutput
+            #endif
+        };
+
         /**
          * @brief Flag
          *
@@ -149,6 +219,12 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
              */
             SpecularTexture = 1 << 2,
 
+            /**
+             * Modify normals according to a texture. Requires the
+             * @ref Tangent attribute to be present.
+             */
+            NormalTexture = 1 << 4,
+
             /**
              * Enable alpha masking. If the combined fragment color has an
              * alpha less than the value specified with @ref setAlphaMask(),
@@ -160,7 +236,24 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
              * with proper depth sorting and blending you'll usually get much
              * better performance and output quality.
              */
-            AlphaMask = 1 << 3
+            AlphaMask = 1 << 3,
+
+            /**
+             * Multiply diffuse color with a vertex color. Requires either
+             * the @ref Color3 or @ref Color4 attribute to be present.
+             */
+            VertexColor = 1 << 5,
+
+            #ifndef MAGNUM_TARGET_GLES2
+            /**
+             * Enable object ID output. See @ref Shaders-Phong-usage-object-id
+             * for more information.
+             * @requires_gles30 Object ID output requires integer buffer
+             *      attachments, which are not available in OpenGL ES 2.0 or
+             *      WebGL 1.0.
+             */
+            ObjectId = 1 << 6
+            #endif
         };
 
         /**
@@ -180,12 +273,14 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
         /**
          * @brief Construct without creating the underlying OpenGL object
          *
-         * The constructed instance is equivalent to moved-from state. Useful
+         * The constructed instance is equivalent to a moved-from state. Useful
          * in cases where you will overwrite the instance later anyway. Move
          * another object over it to make it useful.
          *
          * This function can be safely used for constructing (and later
          * destructing) objects even without any OpenGL context being active.
+         * However note that this is a low-level and a potentially dangerous
+         * API, see the documentation of @ref NoCreate for alternatives.
          */
         explicit Phong(NoCreateT) noexcept: GL::AbstractShaderProgram{NoCreate} {}
 
@@ -216,10 +311,7 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
          * ambient texture, otherwise default value is @cpp 0x00000000_rgbaf @ce.
          * @see @ref bindAmbientTexture()
          */
-        Phong& setAmbientColor(const Color4& color) {
-            setUniform(_ambientColorUniform, color);
-            return *this;
-        }
+        Phong& setAmbientColor(const Magnum::Color4& color);
 
         /**
          * @brief Bind an ambient texture
@@ -244,20 +336,20 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
          * @brief Set diffuse color
          * @return Reference to self (for method chaining)
          *
-         * Initial value is @cpp 0xffffffff_rgbaf @ce.
+         * Initial value is @cpp 0xffffffff_rgbaf @ce. If @ref lightCount() is
+         * zero, this function is a no-op, as diffuse color doesn't contribute
+         * to the output in that case.
          * @see @ref bindDiffuseTexture()
          */
-        Phong& setDiffuseColor(const Color4& color) {
-            setUniform(_diffuseColorUniform, color);
-            return *this;
-        }
+        Phong& setDiffuseColor(const Magnum::Color4& color);
 
         /**
          * @brief Bind a diffuse texture
          * @return Reference to self (for method chaining)
          *
          * Expects that the shader was created with @ref Flag::DiffuseTexture
-         * enabled.
+         * enabled. If @ref lightCount() is zero, this function is a no-op, as
+         * diffuse color doesn't contribute to the output in that case.
          * @see @ref bindTextures(), @ref setDiffuseColor()
          */
         Phong& bindDiffuseTexture(GL::Texture2D& texture);
@@ -271,6 +363,18 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
         }
         #endif
 
+        /**
+         * @brief Bind a normal texture
+         * @return Reference to self (for method chaining)
+         *
+         * Expects that the shader was created with @ref Flag::NormalTexture
+         * enabled and the @ref Tangent attribute was supplied. If
+         * @ref lightCount() is zero, this function is a no-op, as normals
+         * dosn't contribute to the output in that case.
+         * @see @ref bindTextures()
+         */
+        Phong& bindNormalTexture(GL::Texture2D& texture);
+
         /**
          * @brief Set specular color
          * @return Reference to self (for method chaining)
@@ -278,20 +382,20 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
          * Initial value is @cpp 0xffffffff_rgbaf @ce. Color will be multiplied
          * with specular texture if @ref Flag::SpecularTexture is set. If you
          * want to have a fully diffuse material, set specular color to
-         * @cpp 0x000000ff_rgbaf @ce.
+         * @cpp 0x000000ff_rgbaf @ce. If @ref lightCount() is zero, this
+         * function is a no-op, as specular color doesn't contribute to the
+         * output in that case.
          * @see @ref bindSpecularTexture()
          */
-        Phong& setSpecularColor(const Color4& color) {
-            setUniform(_specularColorUniform, color);
-            return *this;
-        }
+        Phong& setSpecularColor(const Magnum::Color4& color);
 
         /**
          * @brief Bind a specular texture
          * @return Reference to self (for method chaining)
          *
          * Expects that the shader was created with @ref Flag::SpecularTexture
-         * enabled.
+         * enabled. If @ref lightCount() is zero, this function is a no-op, as
+         * specular color doesn't contribute to the output in that case.
          * @see @ref bindTextures(), @ref setSpecularColor()
          */
         Phong& bindSpecularTexture(GL::Texture2D& texture);
@@ -312,13 +416,17 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
          * A particular texture has effect only if particular texture flag from
          * @ref Phong::Flag "Flag" is set, you can use @cpp nullptr @ce for the
          * rest. Expects that the shader was created with at least one of
-         * @ref Flag::AmbientTexture, @ref Flag::DiffuseTexture or
-         * @ref Flag::SpecularTexture enabled. More efficient than setting each
-         * texture separately.
+         * @ref Flag::AmbientTexture, @ref Flag::DiffuseTexture,
+         * @ref Flag::SpecularTexture or @ref Flag::NormalTexture enabled. More
+         * efficient than setting each texture separately.
          * @see @ref bindAmbientTexture(), @ref bindDiffuseTexture(),
-         *      @ref bindSpecularTexture()
+         *      @ref bindSpecularTexture(), @ref bindNormalTexture()
          */
-        Phong& bindTextures(GL::Texture2D* ambient, GL::Texture2D* diffuse, GL::Texture2D* specular);
+        Phong& bindTextures(GL::Texture2D* ambient, GL::Texture2D* diffuse, GL::Texture2D* specular, GL::Texture2D* normal
+            #ifdef MAGNUM_BUILD_DEPRECATED
+            = nullptr
+            #endif
+        );
 
         #ifdef MAGNUM_BUILD_DEPRECATED
         /** @brief @copybrief bindTextures()
@@ -334,12 +442,11 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
          * @return Reference to self (for method chaining)
          *
          * The larger value, the harder surface (smaller specular highlight).
-         * Initial value is @cpp 80.0f @ce.
+         * Initial value is @cpp 80.0f @ce. If @ref lightCount() is zero, this
+         * function is a no-op, as specular color doesn't contribute to the
+         * output in that case.
          */
-        Phong& setShininess(Float shininess) {
-            setUniform(_shininessUniform, shininess);
-            return *this;
-        }
+        Phong& setShininess(Float shininess);
 
         /**
          * @brief Set alpha mask value
@@ -352,6 +459,22 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
          */
         Phong& setAlphaMask(Float mask);
 
+        #ifndef MAGNUM_TARGET_GLES2
+        /**
+         * @brief Set object ID
+         * @return Reference to self (for method chaining)
+         *
+         * Expects that the shader was created with @ref Flag::ObjectId
+         * enabled. Value set here is written to the @ref ObjectIdOutput, see
+         * @ref Shaders-Phong-usage-object-id for more information. Default is
+         * @cpp 0 @ce.
+         * @requires_gles30 Object ID output requires integer buffer
+         *      attachments, which are not available in OpenGL ES 2.0 or WebGL
+         *      1.0.
+         */
+        Phong& setObjectId(UnsignedInt id);
+        #endif
+
         /**
          * @brief Set transformation matrix
          * @return Reference to self (for method chaining)
@@ -359,10 +482,7 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
          * You need to set also @ref setNormalMatrix() with a corresponding
          * value. Initial value is an identity matrix.
          */
-        Phong& setTransformationMatrix(const Matrix4& matrix) {
-            setUniform(_transformationMatrixUniform, matrix);
-            return *this;
-        }
+        Phong& setTransformationMatrix(const Matrix4& matrix);
 
         /**
          * @brief Set normal matrix
@@ -371,12 +491,12 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
          * The matrix doesn't need to be normalized, as the renormalization
          * must be done in the shader anyway. You need to set also
          * @ref setTransformationMatrix() with a corresponding value. Initial
-         * value is an identity matrix.
+         * value is an identity matrix. If @ref lightCount() is zero, this
+         * function is a no-op, as normals don't contribute to the output in
+         * that case.
+         * @see @ref Math::Matrix4::normalMatrix()
          */
-        Phong& setNormalMatrix(const Matrix3x3& matrix) {
-            setUniform(_normalMatrixUniform, matrix);
-            return *this;
-        }
+        Phong& setNormalMatrix(const Matrix3x3& matrix);
 
         /**
          * @brief Set projection matrix
@@ -386,10 +506,7 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
          * projection of the default @f$ [ -\boldsymbol{1} ; \boldsymbol{1} ] @f$
          * cube).
          */
-        Phong& setProjectionMatrix(const Matrix4& matrix) {
-            setUniform(_projectionMatrixUniform, matrix);
-            return *this;
-        }
+        Phong& setProjectionMatrix(const Matrix4& matrix);
 
         /**
          * @brief Set light positions
@@ -405,9 +522,7 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
         Phong& setLightPositions(Containers::ArrayView<const Vector3> lights);
 
         /** @overload */
-        Phong& setLightPositions(std::initializer_list<Vector3> lights) {
-            return setLightPositions({lights.begin(), lights.size()});
-        }
+        Phong& setLightPositions(std::initializer_list<Vector3> lights);
 
         /**
          * @brief Set position for given light
@@ -438,12 +553,10 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
          * Initial values are @cpp 0xffffffff_rgbaf @ce. Expects that the size
          * of the @p colors array is the same as @ref lightCount().
          */
-        Phong& setLightColors(Containers::ArrayView<const Color4> colors);
+        Phong& setLightColors(Containers::ArrayView<const Magnum::Color4> colors);
 
         /** @overload */
-        Phong& setLightColors(std::initializer_list<Color4> colors) {
-            return setLightColors({colors.begin(), colors.size()});
-        }
+        Phong& setLightColors(std::initializer_list<Magnum::Color4> colors);
 
         /**
          * @brief Set position for given light
@@ -451,9 +564,9 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
          *
          * Unlike @ref setLightColors() updates just a single light color.
          * Expects that @p id is less than @ref lightCount().
-         * @see @ref setLightColor(const Color4&)
+         * @see @ref setLightColor(const Magnum::Color4&)
          */
-        Phong& setLightColor(UnsignedInt id, const Color4& color);
+        Phong& setLightColor(UnsignedInt id, const Magnum::Color4& color);
 
         /**
          * @brief Set light color
@@ -461,9 +574,9 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
          *
          * Convenience alternative to @ref setLightColors() when there is just
          * one light.
-         * @see @ref setLightColor(UnsignedInt, const Color4&)
+         * @see @ref setLightColor(UnsignedInt, const Magnum::Color4&)
          */
-        Phong& setLightColor(const Color4& color) {
+        Phong& setLightColor(const Magnum::Color4& color) {
             return setLightColors({&color, 1});
         }
 
@@ -477,9 +590,12 @@ class MAGNUM_SHADERS_EXPORT Phong: public GL::AbstractShaderProgram {
             _diffuseColorUniform{5},
             _specularColorUniform{6},
             _shininessUniform{7},
-            _alphaMaskUniform{8},
-            _lightPositionsUniform{9},
-            _lightColorsUniform; /* 9 + lightCount, set in the constructor */
+            _alphaMaskUniform{8};
+            #ifndef MAGNUM_TARGET_GLES2
+            Int _objectIdUniform{9};
+            #endif
+        Int _lightPositionsUniform{10},
+            _lightColorsUniform; /* 10 + lightCount, set in the constructor */
 };
 
 /** @debugoperatorclassenum{Phong,Phong::Flag} */
